/// <reference types="node" />
import { initStore } from "./src/node.polyfill";
import * as entities from "./src/lib/storage/entities";
import * as actions from "./src/actions";
import { BaseMetadata } from 'cred-types-jolocom-core';
import { IdentityWallet } from 'jolocom-lib/js/identityWallet/identityWallet';
import { ICredentialRequest, ICredentialRequestAttrs, CredentialOffer, CredentialOfferRequestAttrs, ICredentialsReceiveAttrs } from 'jolocom-lib/js/interactionTokens/interactionTokens.types';
import { ISignedCredCreationArgs } from 'jolocom-lib/js/credentials/signedCredential/types';
export { initStore, entities, actions };
export { ICredentialRequest as CredentialRequirements, ICredentialRequestAttrs as CredentialRequest, CredentialOffer, CredentialOfferRequestAttrs, ICredentialsReceiveAttrs as CredentialPayload, BaseMetadata as CredentialDefinition, ISignedCredCreationArgs as CredentialData, };
export declare class JolocomSDK {
    private store;
    idw: IdentityWallet;
    constructor(store: ReturnType<typeof initStore>);
    /**
     * Returns an agent with an Identity provided by a store
     *
     * @remarks
     * This depends on an existing database connection
     *
     * @param store - The agent state with storage connection
     * @returns An Agent with the identity existing in the storage in store
     */
    static fromStore(store: ReturnType<typeof initStore>): Promise<JolocomSDK>;
    /**
     * Returns an agent with an Identity provided by a BIP 39 12 word seed phrase
     *
     * @param mnemonic - 12 word BIP 39 seed phrase, space-delimited
     * @returns An Agent with the identity corrosponding to the sead phrase
     */
    static fromMnemonic(mnemonic: string): Promise<JolocomSDK>;
    /**
     * Returns an agent with an Identity provided by a buffer of entropy.
     * WARNING: this registers an identity on the Jolocom DID Method
     *
     * @param seed - Buffer of private entropy to generate keys with
     * @returns An Agent with the identity corrosponding to the seed
     */
    static newDIDFromSeed(seed: Buffer): Promise<JolocomSDK>;
    /**
     * Handles a recieved interaction token
     *
     * @param jwt - recieved jwt, Base64 encoded
     * @returns Promise<bool>, true: valid, false: invalid, reject: incorrect
     */
    tokenRecieved(jwt: string): Promise<boolean>;
    /**
     * Creates a signed, base64 encoded Credential Request, given a set of requirements
     *
     * @param request - Credential Request Attributes
     * @returns Base64 encoded signed credential request
     */
    credRequestToken(request: ICredentialRequestAttrs): Promise<string>;
    /**
     * Returns a base64 encoded signed credential offer token, given
     * request attributes
     *
     * @param offer - credential offer attributes
     * @returns A base64 encoded signed credential offer token offering
     * credentials according to `offer`
     */
    credOfferToken(offer: CredentialOfferRequestAttrs): Promise<string>;
    /**
     * Returns a base64 encoded signed credential issuance token, given
     * issuance attributes and a recieved token selecting desired issuance
     *
     * @param issuance - credential issuance attributes
     * @param selection - base64 encoded credential offer response token
     * @returns A base64 encoded signed issuance token containing verifiable
     * credentials
     */
    credIssuanceToken(issuance: ICredentialsReceiveAttrs, selection: string): Promise<string>;
    /**
     * Returns a Signed Credential
     *
     * @param credParams - credential attributes
     * @returns SignedCredential instance
     */
    signedCredential<T extends BaseMetadata>(credParams: ISignedCredCreationArgs<T>): Promise<import("jolocom-lib/js/credentials/signedCredential/signedCredential").SignedCredential>;
}
