"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const node_polyfill_1 = require("./src/node.polyfill");
exports.initStore = node_polyfill_1.initStore;
const entities = tslib_1.__importStar(require("./src/lib/storage/entities"));
exports.entities = entities;
const actions = tslib_1.__importStar(require("./src/actions"));
exports.actions = actions;
const jolocom_lib_1 = require("jolocom-lib");
const modifiers_1 = require("./src/actions/modifiers");
const types_1 = require("./src/lib/interactionManager/types");
const initErrorHandler = (error) => dispatch => {
    console.error(error.message);
    return Promise.reject(error);
};
// @ts-ignore
// const injectPassFn = (passFn: () => Promise<string>) => <A, T>(
//   delayedFn: (arg1: A, pass: string, ...rest) => Promise<T>,
// ) => async (a1: A, ...rest) => await delayedFn(a1, await passFn())
class JolocomSDK {
    constructor(store) {
        this.store = store;
        this.idw = this.store.backendMiddleware.identityWallet;
    }
    /**
     * Returns an agent with an Identity provided by a store
     *
     * @remarks
     * This depends on an existing database connection
     *
     * @param store - The agent state with storage connection
     * @returns An Agent with the identity existing in the storage in store
     */
    static fromStore(store) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield store.dispatch(modifiers_1.withErrorHandler(initErrorHandler)(actions.accountActions.checkIdentityExists));
            return new JolocomSDK(store);
        });
    }
    /**
     * Returns an agent with an Identity provided by a BIP 39 12 word seed phrase
     *
     * @param mnemonic - 12 word BIP 39 seed phrase, space-delimited
     * @returns An Agent with the identity corrosponding to the sead phrase
     */
    static fromMnemonic(mnemonic) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const store = node_polyfill_1.initStore();
            yield store.dispatch(actions.registrationActions.recoverIdentity(mnemonic));
            return new JolocomSDK(store);
        });
    }
    /**
     * Returns an agent with an Identity provided by a buffer of entropy.
     * WARNING: this registers an identity on the Jolocom DID Method
     *
     * @param seed - Buffer of private entropy to generate keys with
     * @returns An Agent with the identity corrosponding to the seed
     */
    static newDIDFromSeed(seed) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const store = node_polyfill_1.initStore();
            yield store.dispatch(actions.registrationActions.createIdentity(seed.toString('hex')));
            return new JolocomSDK(store);
        });
    }
    /**
     * Handles a recieved interaction token
     *
     * @param jwt - recieved jwt, Base64 encoded
     * @returns Promise<bool>, true: valid, false: invalid, reject: incorrect
     */
    tokenRecieved(jwt) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const token = jolocom_lib_1.JolocomLib.parse.interactionToken.fromJWT(jwt);
            const interaction = this.store.backendMiddleware.interactionManager.getInteraction(token.nonce);
            if (interaction) {
                return interaction.processInteractionToken(token);
            }
            else {
                this.store.backendMiddleware.interactionManager.start(types_1.InteractionChannel.HTTP, token);
                return true;
            }
        });
    }
    /**
     * Creates a signed, base64 encoded Credential Request, given a set of requirements
     *
     * @param request - Credential Request Attributes
     * @returns Base64 encoded signed credential request
     */
    credRequestToken(request) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const token = yield this.idw.create.interactionTokens.request.share(request, yield this.store.backendMiddleware.keyChainLib.getPassword());
            yield this.store.backendMiddleware.interactionManager.start(types_1.InteractionChannel.HTTP, token);
            return token.encode();
        });
    }
    /**
     * Returns a base64 encoded signed credential offer token, given
     * request attributes
     *
     * @param offer - credential offer attributes
     * @returns A base64 encoded signed credential offer token offering
     * credentials according to `offer`
     */
    credOfferToken(offer) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const token = yield this.idw.create.interactionTokens.request.offer(offer, yield this.store.backendMiddleware.keyChainLib.getPassword());
            yield this.store.backendMiddleware.interactionManager.start(types_1.InteractionChannel.HTTP, token);
            return token.encode();
        });
    }
    /**
     * Returns a base64 encoded signed credential issuance token, given
     * issuance attributes and a recieved token selecting desired issuance
     *
     * @param issuance - credential issuance attributes
     * @param selection - base64 encoded credential offer response token
     * @returns A base64 encoded signed issuance token containing verifiable
     * credentials
     */
    credIssuanceToken(issuance, selection) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const token = yield this.idw.create.interactionTokens.response.issue(issuance, yield this.store.backendMiddleware.keyChainLib.getPassword(), jolocom_lib_1.JolocomLib.parse.interactionToken.fromJWT(selection));
            return token.encode();
        });
    }
    /**
     * Returns a Signed Credential
     *
     * @param credParams - credential attributes
     * @returns SignedCredential instance
     */
    signedCredential(credParams) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield this.idw.create.signedCredential(credParams, yield this.store.backendMiddleware.keyChainLib.getPassword());
        });
    }
}
exports.JolocomSDK = JolocomSDK;
