"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const index_1 = require("../index");
const routeList_1 = require("../../routeList");
const account_1 = require("../account");
const checkRecoverySetup_1 = require("../notifications/checkRecoverySetup");
const notifications_1 = require("../../lib/notifications");
const notifications_2 = require("../notifications");
const i18n_1 = tslib_1.__importDefault(require("../../polyfills/i18n"));
const strings_1 = tslib_1.__importDefault(require("../../locales/strings"));
const types_1 = require("../../lib/interactionManager/types");
const ramda_1 = require("ramda");
exports.consumeCredentialOfferRequest = (credentialOfferRequest, interactionChannel) => (dispatch, getState, { interactionManager }) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const interaction = yield interactionManager.start(interactionChannel, credentialOfferRequest);
    return dispatch(index_1.navigationActions.navigate({
        routeName: routeList_1.routeList.CredentialReceive,
        params: {
            interactionId: credentialOfferRequest.nonce,
            interactionSummary: interaction.getSummary(),
        },
    }));
});
/**
 * Given the ID for a {@link CredentialOfferFlow}, and an array representing a user selection,
 * will attempt to create the correct {@link CredentialOfferResponse} token, and pass it to the interaction
 * class instance to move the interaction forward.
 * @todo - interaction.send can throw, this is not currently handled.
 */
exports.consumeCredentialReceive = (selectedSignedCredentialWithMetadata, interactionId) => (dispatch, getState, { interactionManager }) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const interaction = interactionManager.getInteraction(interactionId);
    const credentialReceive = yield interaction.send(yield interaction.createCredentialOfferResponseToken(selectedSignedCredentialWithMetadata));
    if (credentialReceive) {
        yield interaction.processInteractionToken(credentialReceive);
        return dispatch(exports.validateSelectionAndSave(selectedSignedCredentialWithMetadata, interaction.id));
    }
});
/**
 * Given the ID for a {@link CredentialOfferFlow}, and an array representing a user selection,
 * will ensure that the user selection is sane (i.e. the user selected credentials from the offer),
 * and navigate accordingly.
 * In case the selection is sane, we try store the selected credentials and navigate to home.
 * In case the selection contains SOME entries which can't be saved (i.e. wrong issuer / subject, already stored),
 * we navigate to the selection screen again, with the problematic credentials marked as invalid (and therefore unselectable)
 * In case the selection contains NO entries that can be saved, a error notification is rendered, and the interaction is ended.
 *
 * @dev This function can eventually be simplified. It currently contains a bunch of logic which is not well abstracted / does not fit anywhere else
 * (e.g. checking for duplicates)
 */
exports.validateSelectionAndSave = (selectedCredentials, interactionId) => (dispatch, getState, { interactionManager, storageLib }) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const interaction = interactionManager.getInteraction(interactionId);
    const offerSummary = interaction.flow.getIssuanceResult();
    const selectedTypes = selectedCredentials.map(el => el.type);
    const toSave = offerSummary.filter(el => selectedTypes.includes(el.type));
    if (toSave.length !== selectedCredentials.length) {
        // TODO Decide how to handle this
        // Means one of the selections isn't in the offer
    }
    const duplicates = yield isCredentialStored(toSave, id => interaction.getStoredCredentialById(id));
    const validationErrors = toSave.map(({ validationErrors }, i) => validationErrors.invalidIssuer ||
        !!validationErrors.invalidSubject ||
        duplicates[i]);
    // None passed the validation
    const allInvalid = !validationErrors.includes(false);
    // At least one passed the validatio
    const allValid = !validationErrors.includes(true);
    const scheduleInvalidNotification = (message) => notifications_2.scheduleNotification(notifications_1.createInfoNotification({
        title: i18n_1.default.t(strings_1.default.AWKWARD),
        message,
    }));
    if (allInvalid) {
        dispatch(scheduleInvalidNotification(i18n_1.default.t(strings_1.default.IT_SEEMS_LIKE_WE_CANT_DO_THIS)));
        return dispatch(endReceiving(interactionId));
    }
    if (allValid) {
        yield interaction.storeCredential(toSave);
        yield storeOfferMetadata(interaction.getSummary().state.offerSummary, interaction.participants.requester.did, storageLib.store.credentialMetadata);
        yield interaction.storeIssuerProfile();
        dispatch(checkRecoverySetup_1.checkRecoverySetup);
        //TODO @mnzaki can we avoid running the FULL setClaimsForDid
        dispatch(account_1.setClaimsForDid);
        const notification = notifications_1.createInfoNotification({
            title: i18n_1.default.t(strings_1.default.GREAT_SUCCESS),
            message: i18n_1.default.t(strings_1.default.YOU_CAN_FIND_YOUR_NEW_CREDENTIAL_IN_THE_DOCUMENTS),
            interact: {
                label: i18n_1.default.t(strings_1.default.OPEN),
                onInteract: () => {
                    dispatch(index_1.navigationActions.navigate({ routeName: routeList_1.routeList.Documents }));
                },
            },
        });
        dispatch(notifications_2.scheduleNotification(notification));
        return dispatch(endReceiving(interactionId));
    }
    dispatch(scheduleInvalidNotification(i18n_1.default.t(strings_1.default.SOMETHING_WENT_WRONG_CHOOSE_AGAIN)));
    // The screen is borked. Save is enabled by default. Not sure what's wrong
    return dispatch(index_1.navigationActions.navigate({
        routeName: routeList_1.routeList.CredentialReceiveNegotiate,
        params: {
            interactionId,
            interactionSummary: Object.assign(Object.assign({}, interaction.getSummary()), { state: { offerSummary } }),
        },
    }));
});
/**
 * Helper function to check if a credential with a given ID is already stored in the wallet,
 * used to detect "duplicate" offers
 */
const isCredentialStored = (offer, getCredential) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    return Promise.all(offer.map(({ signedCredential }) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        return signedCredential
            ? !ramda_1.isEmpty(yield getCredential(signedCredential.id))
            : false;
    })));
});
/**
 * @todo currently the only helper which takes a function as an arg instead of a reference to the (in this instance)
 * storage class. Does this need to change for consistency?
 */
const storeOfferMetadata = (offer, did, storeCredentialMetadata) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    return Promise.all(ramda_1.uniqBy(detail => `${detail.issuer.did}${detail.type}`, offer.map(({ type, renderInfo, metadata }) => ({
        // TODO Why isn't the did already in the summary type? This feels hacky
        issuer: { did },
        type,
        renderInfo: renderInfo || {},
        metadata: metadata || {},
    }))).map(storeCredentialMetadata));
});
const endReceiving = (interactionId) => (dispatch, getState, { interactionManager }) => {
    const interaction = interactionManager.getInteraction(interactionId);
    const { channel } = interaction;
    if (channel === types_1.InteractionChannel.Deeplink) {
        return dispatch(index_1.navigationActions.navigatorResetHome());
    }
    else {
        return dispatch(index_1.navigationActions.navigate({ routeName: routeList_1.routeList.InteractionScreen }));
    }
};
